"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const command_line_usage_1 = __importDefault(require("command-line-usage"));
const AbstractCommand_1 = require("./AbstractCommand");
const Cli_1 = require("../Cli");
const decorators_1 = require("../ServiceManager/decorators");
class HelpCommand extends AbstractCommand_1.AbstractCommand {
    output(output, params) {
        const config = this.cliService.getConfig();
        const programs = this.cliService.getPrograms();
        if (params.command) {
            const command = this.cliService.getCommand(params.command);
            if (!command) {
                return output.error(`Unknown command "${params.command}".`);
            }
            return output.addData(command_line_usage_1.default(this.renderUsage(command)));
        }
        const sections = [
            { header: config.title, content: `{italic ${config.subtitle}}` },
            {
                header: chalk_1.default.yellow('Available commands:'),
            },
        ];
        const collectedExamples = [];
        Object.values(programs).forEach(({ program, commands, examples }) => {
            sections.push({ content: chalk_1.default.yellow(program), raw: true });
            sections.push({ content: Object.keys(commands).map((command) => this.renderCommand(commands[command])) });
            if (Array.isArray(examples)) {
                collectedExamples.push(...examples);
            }
        });
        if (collectedExamples) {
            sections.push(this.renderExamples(collectedExamples));
        }
        output.addData(command_line_usage_1.default(sections));
    }
    renderUsage(command) {
        const sections = [
            { header: chalk_1.default.yellow('Usage'), content: [this.renderCommand(command)] },
        ];
        if (command.config) {
            if (command.config.options) {
                sections.push(this.renderOptions(command.config.options));
            }
            if (command.config.examples) {
                sections.push(this.renderExamples(command.config.examples));
            }
        }
        return sections;
    }
    renderOptions(options) {
        return {
            header: chalk_1.default.yellow('Options:'),
            optionList: Object.keys(options).map(optionName => {
                const { alias, description, value } = options[optionName];
                return {
                    description,
                    alias: chalk_1.default.green(alias),
                    name: chalk_1.default.green(optionName),
                    typeLabel: value && `{underline ${value}}`,
                };
            }),
        };
    }
    renderExamples(examples) {
        return {
            header: chalk_1.default.yellow('Examples:'),
            content: examples.map(description => ({ description })),
        };
    }
    renderCommand({ commandLine, config }) {
        return { name: chalk_1.default.green(commandLine), description: (config && config.description) || '{italic No description found}' };
    }
}
__decorate([
    decorators_1.inject(Cli_1.CliService),
    __metadata("design:type", Cli_1.CliService)
], HelpCommand.prototype, "cliService", void 0);
exports.HelpCommand = HelpCommand;
//# sourceMappingURL=HelpCommand.js.map