"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ModuleManagerEvents_1 = require("./ModuleManagerEvents");
const Application_1 = require("../Application");
const debug_1 = require("../../debug");
const debug = debug_1.createDebugLogger('modules');
class ModuleManager {
    constructor(application, eventManager, config) {
        this.application = application;
        this.eventManager = eventManager;
        this.config = config;
    }
    async bootstrap() {
        return await this.eventManager.trigger(ModuleManagerEvents_1.ModuleManagerEvents.OnBootstrap, this);
    }
    async loadModule(ModuleClass) {
        debug('Loading module ' + ModuleClass.name);
        const mode = this.config.of('application').mode;
        const eventManager = this.eventManager;
        const module = new ModuleClass();
        if (typeof module.getConfig === 'function') {
            this.config.merge(await module.getConfig(mode));
        }
        if (mode === Application_1.ApplicationModes.Cli && typeof module.getCliConfig === 'function') {
            this.config.merge(await module.getCliConfig());
        }
        if (mode === Application_1.ApplicationModes.Server && typeof module.getServerConfig === 'function') {
            this.config.merge(await module.getServerConfig());
        }
        if (typeof module.init === 'function') {
            await module.init(this);
        }
        // Allow for a convenience onBootstrap method.
        if (typeof module.onBootstrap === 'function' && !eventManager.has(ModuleManagerEvents_1.ModuleManagerEvents.OnBootstrap, module.onBootstrap)) {
            debug(`Auto-attaching onBootstrap listener for ${ModuleClass.name}.`);
            eventManager.attachOnce(ModuleManagerEvents_1.ModuleManagerEvents.OnBootstrap, module.onBootstrap);
        }
        debug('Initialized module ' + ModuleClass.name);
        return this;
    }
    getEventManager() {
        return this.eventManager;
    }
    getApplication() {
        return this.application;
    }
    async loadModules(config) {
        debug('Loading modules');
        if (!config) {
            debug('No modules registered.');
            return;
        }
        for (let i = 0; i < config.length; i++) {
            await this.loadModule(config[i]);
        }
        debug('Loaded modules');
        return this;
    }
}
exports.ModuleManager = ModuleManager;
//# sourceMappingURL=ModuleManager.js.map