export declare class Output {
    private exitCode;
    private data;
    private prettyError;
    static create(): Output;
    static errorOutput(error?: string | Error, exitCode?: number): Output;
    /**
     * Exit code to use upon send.
     *
     *  NOTE: Exit codes 1 - 2, 126 - 165, and 255 have special meanings
     *        and should therefore be avoided for user-specified exit parameters.
     *        Try restricting user-defined exit codes to the range 64 - 113 (in addition to 0, for success).
     *
     * 1 - Catchall for general errors
     * 2 - Misuse of shell builtins (according to Bash documentation)
     * 126 - Command invoked cannot execute
     * 127 - “command not found”
     * 128 - Invalid argument to exit
     * 128+n - Fatal error signal “n”
     * 130 - Script terminated by Control-C
     * 255\* - Exit status out of range
     *
     * @see http://tldp.org/LDP/abs/html/exitcodes.html
     *
     * @param {Number} exitCode
     *
     * @return {this}
     */
    setExitCode(exitCode: number): this;
    addData(...data: any[]): this;
    success(message: string, data?: any): void;
    resetData(): this;
    addHorizontalTable(head: string[], data: Array<string>[], options?: any): this;
    addVerticalTable(data: {
        [key: string]: string;
    }[], options?: any): this;
    addCrossTable(head: string[], data: {
        [key: string]: string[];
    }[], options?: any): this;
    /**
     * Add/set an error to output.
     *
     * @param {Error|string}  [error]     The error to write.         Default to "Unknown error".
     * @param {Number|null}   [exitCode]  The code to exit with.      Default to 1. Use null to not change the exitCode.
     * @param {boolean}       [clear]     Clear previously set data.  Default to false.
     *
     * @return {this}
     */
    error(error?: Error | string, exitCode?: number, clear?: boolean): this;
    /**
     * Write the output to the console.
     */
    flush(): void;
    send(): void;
}
