"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_send_1 = __importDefault(require("koa-send"));
const path_1 = __importDefault(require("path"));
const ResponseStrategies_1 = require("./ResponseStrategies");
class Response {
    constructor({ message, data, meta, statusCode }) {
        this.headers = {};
        this.strategy = ResponseStrategies_1.ResponseStrategies.Json;
        this.data = data;
        this.meta = meta;
        this.message = message;
        this.statusCode = statusCode;
    }
    setData(data) {
        this.data = data;
        return this;
    }
    getData() {
        return this.data;
    }
    async patchContext(ctx) {
        this.ctx = ctx;
        // Basic stuff
        this.applyStatusCode(this.statusCode).applyHeaders();
        if (this.strategy === ResponseStrategies_1.ResponseStrategies.File) {
            await this.sendFile();
        }
        else if (this.strategy === ResponseStrategies_1.ResponseStrategies.Html) {
            ctx.type = 'html';
            this.applyBody(this.data);
        }
        else {
            // Default to json.
            this.applyBody(this.format());
        }
        // Allow hooks.
        if (typeof this.apply === 'function') {
            await this.apply();
        }
    }
    async sendFile() {
        const { location, options = {} } = this.meta;
        const { dir, base } = path_1.default.parse(location);
        if (options.root) {
            return await koa_send_1.default(this.ctx, location, options);
        }
        await koa_send_1.default(this.ctx, base, Object.assign({ root: dir }, options));
    }
    applyStatusCode(statusCode) {
        this.ctx.status = statusCode;
        return this;
    }
    setStatusCode(statusCode) {
        this.statusCode = statusCode;
        return this;
    }
    getStatusCode() {
        return this.statusCode;
    }
    applyBody(body) {
        if (body) {
            this.ctx.body = body;
        }
        return this;
    }
    setHeaders(headers) {
        this.headers = headers;
        return this;
    }
    addHeaders(headers) {
        Reflect.ownKeys(headers).forEach((header) => this.setHeader(header, headers[header]));
        return this;
    }
    setHeader(header, value) {
        this.headers[header] = value;
        return this;
    }
    appendHeader(header, value) {
        this.headers[header] = [].concat(this.headers[header], value);
    }
    removeHeader(header) {
        delete this.headers[header];
        return this;
    }
    applyHeaders() {
        Reflect.ownKeys(this.headers).forEach((header) => this.ctx.set(header, this.headers[header]));
        return this;
    }
    file(location, options) {
        this.strategy = ResponseStrategies_1.ResponseStrategies.File;
        this.meta = { location, options };
        return this;
    }
    json(data) {
        this.strategy = ResponseStrategies_1.ResponseStrategies.Json;
        this.data = data;
        return this;
    }
    html(data) {
        this.strategy = ResponseStrategies_1.ResponseStrategies.Html;
        this.data = data;
        return this;
    }
    apply() {
        return;
    }
    format() {
        return;
    }
}
exports.Response = Response;
//# sourceMappingURL=Response.js.map