"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_to_regexp_1 = __importDefault(require("path-to-regexp"));
class RouterService {
    constructor(config) {
        this.routes = [];
        this.config = config;
        this.registerRoutes(this.config.routes);
    }
    resolve(method, target) {
        for (let i = 0; i < this.routes.length; i++) {
            const route = this.routes[i];
            if (method !== route.method) {
                continue;
            }
            const parameters = this.match(target, route);
            if (parameters) {
                return { route, parameters };
            }
        }
        return null;
    }
    match(target, route) {
        const { regex, keys } = route;
        const result = regex.exec(target);
        if (!result) {
            return null;
        }
        // We don't care about the path.
        result.splice(0, 1);
        return this.buildParameters(keys, result);
    }
    registerRoutes(routes) {
        routes.forEach(newRoute => {
            if (Array.isArray(newRoute)) {
                return this.registerRoutes(newRoute);
            }
            const { method, route, action, controller } = newRoute;
            this.registerRoute(method, route, controller, action);
        });
        return this;
    }
    registerRoute(method, route, controller, action) {
        const keys = [];
        const regex = path_to_regexp_1.default(route, keys);
        // Remove route if previously registered.
        const routeIndex = this.routes.findIndex(target => target.route === route && target.method === method);
        if (routeIndex > -1) {
            this.routes.splice(routeIndex, 1);
        }
        this.routes.push({ regex, controller, action, keys, method, route });
        return this;
    }
    getRegisteredRoutes() {
        return this.routes;
    }
    buildParameters(from, result) {
        return result.reduce((params, match, index) => {
            return Object.assign(params, { [from[index].name]: match });
        }, {});
    }
}
exports.RouterService = RouterService;
//# sourceMappingURL=RouterService.js.map