"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_1 = __importDefault(require("koa"));
const url_1 = __importDefault(require("url"));
const https_1 = __importDefault(require("https"));
const Application_1 = require("../Application");
const Middleware_1 = require("../Middleware");
class ServerService {
    constructor(mode, config, middleware) {
        this.middleware = [];
        this.config = config;
        if (mode === Application_1.ApplicationModes.Server) {
            this.server = new koa_1.default();
            this.middleware = this.server.middleware;
        }
        this.use(...middleware);
    }
    use(...middlewares) {
        middlewares.forEach(middleware => {
            this.middleware.push(this.asCallback(middleware));
        });
        return this;
    }
    useBefore(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware), 0, ...middlewares);
    }
    useAfter(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware) + 1, 0, ...middlewares);
    }
    replace(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware), 1, ...middlewares);
    }
    updateMiddleware(at, remove, ...middlewares) {
        this.middleware.splice(at === -1 ? 0 : at, remove, ...middlewares.map(middleware => this.asCallback(middleware)));
        return this;
    }
    indexOfMiddleware(middleware) {
        if (!this.server) {
            return -1;
        }
        return this.middleware.findIndex((suspect) => {
            if (typeof middleware === 'string') {
                if (suspect._name) {
                    return suspect._name === middleware;
                }
                return suspect.name === middleware;
            }
            if (suspect._fromClass) {
                return middleware === suspect._fromClass;
            }
            return suspect === middleware;
        });
    }
    getServer() {
        return this.server;
    }
    getURL() {
        const { port, ssl, hostname } = this.config;
        return url_1.default.format(`http${ssl ? 's' : ''}://${hostname}:${port == 80 ? '' : port}`);
    }
    start() {
        if (this.config.ssl) {
            https_1.default.createServer(this.config.ssl, this.server.callback()).listen(this.config.port);
        }
        else {
            this.server.listen(this.config.port);
        }
        return this;
    }
    asCallback(middleware) {
        return middleware instanceof Middleware_1.AbstractMiddleware ? middleware.asCallback() : middleware;
    }
}
exports.ServerService = ServerService;
//# sourceMappingURL=ServerService.js.map