"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const metaKey = Symbol('stix:di:patch');
exports.patch = (name, method, factory = false) => {
    return (target) => {
        const meta = Reflect.getMetadata(metaKey, target) || [];
        meta.push({ name, method, factory });
        Reflect.defineMetadata(metaKey, meta, target);
    };
};
exports.applyPatches = (sm, target) => {
    const patches = Reflect.getMetadata(metaKey, target.constructor);
    if (!patches) {
        return target;
    }
    return patches.reduce((target, { name, method, factory }) => {
        if (typeof target[name] !== 'function') {
            target[name] = (factory ? method(sm, target) : method).bind(target);
        }
        return target;
    }, target);
};
//# sourceMappingURL=patch.js.map